@@ -3,8 +3,9 @@
  * Copyright 1999,2000,2001 BrightStation PLC
  * Copyright 2001,2005 James Aylett
  * Copyright 2001,2002 Ananova Ltd
- * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2011 Olly Betts
+ * Copyright 2002,2003,2004,2005,2006,2007,2008,2009,2010,2011,2012 Olly Betts
  * Copyright 2009 Frank J Bruzzaniti
+ * Copyright 2012 Mihai Bivol
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -43,6 +44,7 @@
 
 #include <xapian.h>
 
+#include "atomparse.h"
 #include "commonhelp.h"
 #include "diritor.h"
 #include "hashterm.h"
@@ -80,6 +82,7 @@ static bool skip_duplicates = false;
 static bool follow_symlinks = false;
 static bool ignore_exclusions = false;
 static bool spelling = false;
+static off_t  max_size = 0;
 static bool verbose = false;
 static enum {
     EMPTY_BODY_WARN, EMPTY_BODY_INDEX, EMPTY_BODY_SKIP
@@ -382,6 +385,11 @@ index_file(const string &file, const string &url, DirectoryIterator & d,
 	return;
     }
 
+    if (max_size > 0 && d.get_size() > max_size) {
+	skip(file, "Larger than size limit", SKIP_VERBOSE_ONLY);
+	return;
+    }
+
     string urlterm("U");
     urlterm += url;
 
@@ -769,6 +777,13 @@ index_file(const string &file, const string &url, DirectoryIterator & d,
 	    if (idx != string::npos) {
 		dump.assign(desc, idx + 1, string::npos);
 	    }
+	} else if (mimetype == "application/atom+xml") {
+	    AtomParser atomparser;
+	    atomparser.parse_html(d.file_to_string());
+	    dump = atomparser.dump;
+	    title = atomparser.title;
+	    keywords = atomparser.keywords;
+	    author = atomparser.author;
 	} else {
 	    // Don't know how to index this type.
 	    skip_unknown_mimetype(file, mimetype);
@@ -932,7 +947,7 @@ index_file(const string &file, const string &url, DirectoryIterator & d,
 		}
 	    }
 	    if (verbose) {
-		if (did < old_lastdocid) {
+		if (did <= old_lastdocid) {
 		    cout << "updated" << endl;
 		} else {
 		    cout << "added" << endl;
@@ -1030,6 +1045,7 @@ main(int argc, char **argv)
 	{ "spelling",	no_argument,		NULL, 'S' },
 	{ "verbose",	no_argument,		NULL, 'v' },
 	{ "empty-docs",	required_argument,	NULL, 'e' },
+	{ "max-size",	required_argument,	NULL, 'm' },
 	{ 0, 0, NULL, 0 }
     };
 
@@ -1156,17 +1172,34 @@ main(int argc, char **argv)
     // RPM packages:
     mime_map["rpm"] = "application/x-redhat-package-manager";
 
+    // Atom feeds:
+    mime_map["atom"] = "application/atom+xml";
+
     // Extensions to quietly ignore:
     mime_map["a"] = "ignore";
+    mime_map["bin"] = "ignore";
+    mime_map["css"] = "ignore";
+    mime_map["dat"] = "ignore";
+    mime_map["db"] = "ignore";
     mime_map["dll"] = "ignore";
     mime_map["dylib"] = "ignore";
     mime_map["exe"] = "ignore";
+    mime_map["fon"] = "ignore";
+    mime_map["jar"] = "ignore";
+    mime_map["js"] = "ignore";
     mime_map["lib"] = "ignore";
+    mime_map["lnk"] = "ignore";
     mime_map["o"] = "ignore";
     mime_map["obj"] = "ignore";
+    mime_map["pyc"] = "ignore";
+    mime_map["pyd"] = "ignore";
+    mime_map["pyo"] = "ignore";
     mime_map["so"] = "ignore";
-    mime_map["css"] = "ignore";
-    mime_map["js"] = "ignore";
+    mime_map["sqlite"] = "ignore";
+    mime_map["sqlite3"] = "ignore";
+    mime_map["sqlite-journal"] = "ignore";
+    mime_map["tmp"] = "ignore";
+    mime_map["ttf"] = "ignore";
 
     commands["application/msword"] = "antiword -mUTF-8.txt ";
     commands["application/vnd.ms-powerpoint"] = "catppt -dutf-8 ";
@@ -1192,7 +1225,7 @@ main(int argc, char **argv)
 
     string dbpath;
     int getopt_ret;
-    while ((getopt_ret = gnu_getopt_long(argc, argv, "hvd:D:U:M:F:l:s:pfSVe:i",
+    while ((getopt_ret = gnu_getopt_long(argc, argv, "hvd:D:U:M:F:l:s:pfSVe:im:",
 					 longopts, NULL)) != -1) {
 	switch (getopt_ret) {
 	case 'h': {
@@ -1222,6 +1255,8 @@ main(int argc, char **argv)
 "  -f, --follow             follow symbolic links\n"
 "  -i, --ignore-exclusions  ignore meta robots tags and similar exclusions\n"
 "  -S, --spelling           index data for spelling correction\n"
+"  -m, --max-size           maximum size of file to index (in bytes or with a\n"
+"                           suffix of 'K'/'k', 'M'/'m', 'G'/'g')\n"
 "  -v, --verbose            show more information about what is happening\n"
 "      --overwrite          create the database anew (the default is to update\n"
 "                           if the database already exists)" << endl;
@@ -1326,6 +1361,35 @@ main(int argc, char **argv)
 	case 'v':
 	    verbose = true;
 	    break;
+	case 'm': {
+	    // Don't want negative numbers, infinity, NaN, or hex numbers.
+	    char * p = optarg;
+	    if (C_isdigit(p[0]) && (p[1] | 32) != 'x') {
+		double arg = strtod(p, &p);
+		switch (*p) {
+		    case '\0':
+			break;
+		    case 'k': case 'K':
+			arg *= 1024;
+			++p;
+			break;
+		    case 'm': case 'M':
+			arg *= (1024 * 1024);
+			++p;
+			break;
+		    case 'g': case 'G':
+			arg *= (1024 * 1024 * 1024);
+			++p;
+			break;
+		}
+		if (*p == '\0') {
+		    max_size = off_t(arg);
+		    break;
+		}
+	    }
+	    cerr << PROG_NAME": bad max size '" << optarg << "'" << endl;
+	    return 1;
+	}
 	case ':': // missing param
 	    return 1;
 	case '?': // unknown option: FIXME -> char
@@ -1459,6 +1523,8 @@ main(int argc, char **argv)
 	exitcode = 0;
     } catch (const Xapian::Error &e) {
 	cout << "Exception: " << e.get_description() << endl;
+    } catch (const exception &e) {
+	cout << "Exception: " << e.what() << endl;
     } catch (const string &s) {
 	cout << "Exception: " << s << endl;
     } catch (const char *s) {
